@@ -42,7 +42,7 @@ import urllib.request
 from py_tools_ds.processing.progress_mon import ProgressBar
 
 from sicor.Tools.EnMAP.metadata import varsol
-from sicor.Tools.EnMAP.LUT import get_data_file, read_lut_enmap_formatted, interpol_lut_c
+from sicor.Tools.EnMAP.LUT import get_data_file, read_lut_enmap_formatted, interpol_lut_c, download_LUT
 from sicor.Tools.EnMAP.conversion import generate_filter, table_to_array
 from sicor.Tools.EnMAP.optimal_estimation import invert_function
 from sicor.Tools.EnMAP.multiprocessing import SharedNdarray, initializer, mp_progress_bar
@@ -171,13 +171,13 @@ class FoGen(object):
             else:
                 self.logger.info("LUT file was not found locally. Try to download it from Git repository...")
                 self.fname =\
-                    "https://gitext.gfz-potsdam.de/EnMAP/sicor/-/raw/master/sicor/AC/data/EnMAP_LUT_MOD5_formatted_1nm"
+                    "https://git.gfz-potsdam.de/EnMAP/sicor/-/raw/master/sicor/AC/data/EnMAP_LUT_MOD5_formatted_1nm"
                 urllib.request.urlretrieve(self.fname, self.path_LUT_default)
                 if os.path.isfile(self.path_LUT_default):
                     self.fn_table = self.path_LUT_default
                 else:
                     raise FileNotFoundError('Download of LUT file failed. Please download it manually from'
-                                            'https://gitext.gfz-potsdam.de/EnMAP/sicor and store it at /sicor/AC/data/'
+                                            'https://git.gfz-potsdam.de/EnMAP/sicor and store it at /sicor/AC/data/'
                                             'directory. Otherwise, the AC will not work.')
 
         # fn_table = get_data_file(module_name="sicor", file_basename="EnMAP_LUT_MOD5_formatted_1nm")
@@ -214,7 +214,7 @@ class FoGen(object):
         # load imaginary parts of refractive indexes of liquid water and ice
         self.logger.info("load imaginary parts of refractive indexes of liquid water and ice...")
         path_k = get_data_file(module_name="sicor", file_basename="k_liquid_water_ice.xlsx")
-        k_wi = pd.read_excel(io=path_k)
+        k_wi = pd.read_excel(io=path_k, engine='openpyxl')
         wvl_water, k_water = table_to_array(k_wi=k_wi, a=0, b=982, col_wvl="wvl_6", col_k="T = 20°C")
         interp_kw = np.interp(x=self.wvl_sel, xp=wvl_water, fp=k_water)
         self.kw = interp_kw
@@ -331,6 +331,16 @@ class Fo(object):
         self.instrument = "EnMAP"
         self.data = enmap_l1b.swir.data
         self.data_fg = enmap_l1b.vnir.data
+        self.land_only = options["EnMAP"]["Retrieval"]["land_only"]
+
+        if self.land_only:
+            logger.info("SICOR is applied to land pixels only. This may result in edge effects, e.g., at coastlines...")
+            self.water_mask = enmap_l1b.vnir.mask_landwater[:, :]
+            self.data[self.water_mask != 1] = 0.0
+            self.data_fg[self.water_mask != 1] = 0.0
+        else:
+            logger.info("SICOR is applied to land AND water pixels.")
+
         self.enmap_rad_all = np.concatenate((self.data_fg, self.data), axis=2)
 
         # get observation metadata
@@ -451,23 +461,15 @@ class Fo(object):
                 self.fn_table = self.path_LUT_default
             else:
                 self.logger.info("LUT file was not found locally. Try to download it from Git repository...")
-                self.fname = \
-                    "https://gitext.gfz-potsdam.de/EnMAP/sicor/-/raw/master/sicor/AC/data/EnMAP_LUT_MOD5_formatted_1nm"
-                urllib.request.urlretrieve(self.fname, self.path_LUT_default)
-                if os.path.isfile(self.path_LUT_default):
-                    self.fn_table = self.path_LUT_default
-                else:
-                    raise FileNotFoundError('Download of LUT file failed. Please download it manually from'
-                                            'https://gitext.gfz-potsdam.de/EnMAP/sicor and store it at /sicor/AC/data/'
-                                            'directory. Otherwise, the AC will not work.')
+                self.fn_table = download_LUT(path_LUT_default=self.path_LUT_default)
 
         # check if LUT is a regular file (not only an LFS pointer)
         try:
             assert os.path.getsize(self.fn_table) > 1000
             self.logger.info("LUT file was properly downloaded and is available for AC!")
         except AssertionError:
-            raise AssertionError("LUT file only represents an LFS pointer. Make sure to download the file following "
-                                 "the installation documentation on Git. Otherwise, the AC will not work.")
+            self.logger.info("LUT file only represents an LFS pointer. Try to download it from Git repository...")
+            self.fn_table = download_LUT(path_LUT_default=self.path_LUT_default)
 
         luts, axes_x, axes_y, wvl, lut1, lut2, xnodes, nm_nodes, ndim, x_cell = read_lut_enmap_formatted(
             file_lut=self.fn_table)
@@ -508,7 +510,7 @@ class Fo(object):
         warnings.filterwarnings("ignore")
         self.logger.info("Loading imaginary parts of refractive indexes of liquid water and ice...")
         path_k = get_data_file(module_name="sicor", file_basename="k_liquid_water_ice.xlsx")
-        k_wi = pd.read_excel(io=path_k)
+        k_wi = pd.read_excel(io=path_k, engine='openpyxl')
         wvl_water, k_water = table_to_array(k_wi=k_wi, a=0, b=982, col_wvl="wvl_6", col_k="T = 20°C")
         interp_kw = np.interp(x=self.wvl_sel, xp=wvl_water, fp=k_water)
         self.kw = interp_kw
@@ -528,6 +530,7 @@ class Fo(object):
         self.segmentation = options["EnMAP"]["Retrieval"]["segmentation"]
         self.n_pca = options["EnMAP"]["Retrieval"]["n_pca"]
         self.segs = options["EnMAP"]["Retrieval"]["segs"]
+
         if self.segmentation:
             self.logger.info("Segmenting L1B spectra to enhance processing speed...")
             self.X, self.segs, self.labels = SLIC_segmentation(data_rad_all=self.enmap_rad_all, n_pca=self.n_pca,
@@ -535,16 +538,27 @@ class Fo(object):
 
             # prepare segmented L1B data cube
             self.logger.info("Preparing segmented L1B data cube...")
+            if self.land_only:
+                self.labels[self.water_mask != 1] = -1.0
+                self.lbl = np.unique(self.labels)[1:]
+                self.segs = len(self.lbl)
+
             self.rdn_subset = np.zeros((1, self.segs, self.enmap_rad_all.shape[2]))
             self.dem_subset = np.zeros((1, self.segs))
             self.pt_subset = np.zeros((1, self.segs, 5))
-            for i in range(self.segs):
-                radiance = self.X[self.labels.flat == i, :].mean(axis=0)
-                hsf_mean = self.hsf["swir"].flatten()[self.labels.flat == i].mean(axis=0)
-                pt_mean = self.pt.reshape(self.pt.shape[0] * self.pt.shape[1], 5)[self.labels.flat == i, :].mean(axis=0)
-                self.rdn_subset[:, i, :] = radiance
-                self.dem_subset[:, i] = hsf_mean
-                self.pt_subset[:, i, :] = pt_mean
+
+            if self.land_only:
+                for ii, lbl in enumerate(self.lbl):
+                    self.rdn_subset[:, ii, :] = self.X[self.labels.flat == lbl, :].mean(axis=0)
+                    self.dem_subset[:, ii] = self.hsf["swir"].flatten()[self.labels.flat == lbl].mean(axis=0)
+                    self.pt_subset[:, ii, :] = self.pt.reshape(self.pt.shape[0] * self.pt.shape[1], 5)[
+                                               self.labels.flat == lbl, :].mean(axis=0)
+            else:
+                for ii in range(self.segs):
+                    self.rdn_subset[:, ii, :] = self.X[self.labels.flat == ii, :].mean(axis=0)
+                    self.dem_subset[:, ii] = self.hsf["swir"].flatten()[self.labels.flat == ii].mean(axis=0)
+                    self.pt_subset[:, ii, :] = self.pt.reshape(self.pt.shape[0] * self.pt.shape[1], 5)[
+                                              self.labels.flat == ii, :].mean(axis=0)
 
     def rho_beer_lambert(self, xx):
         """
@@ -717,6 +731,10 @@ def __minimize__(fo, opt_func, logger=None, oe=False, ice=False):
 
     globs = dict()
     globs["__instrument__"] = fo.instrument
+    globs["__land_only__"] = fo.land_only
+    if fo.land_only:
+        globs["__water_mask__"] = fo.water_mask
+    globs["__segmentation__"] = fo.segmentation
 
     if fo.segmentation:
         opt_pt = np.array(fo.pt_subset)
@@ -872,16 +890,23 @@ def __min__(ii):
 
     i1, i2 = ii
 
-    ress_full = minimize(fun=__forward__, x0=__xa__[i1, i2, :], args=(__pt__[i1, i2, :], __data__[i1, i2, :]),
-                         jac=False, method='Nelder-Mead', options={"disp": False})
+    if not __segmentation__ and __land_only__ and __water_mask__[i1, i2] != 1:
+        __cwv_model__[i1, i2] = np.nan
+        __cwc_model__[i1, i2] = np.nan
+        __a_model__[i1, i2] = np.nan
+        __b_model__[i1, i2] = np.nan
+        __toa_model__[i1, i2, :] = np.full(len(__fit_wvl__), np.nan)
+    else:
+        ress_full = minimize(fun=__forward__, x0=__xa__[i1, i2, :], args=(__pt__[i1, i2, :], __data__[i1, i2, :]),
+                             jac=False, method='Nelder-Mead', options={"disp": False})
 
-    model = __forward__(xx=ress_full["x"], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
+        model = __forward__(xx=ress_full["x"], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
 
-    __cwv_model__[i1, i2] = ress_full["x"][0]
-    __cwc_model__[i1, i2] = ress_full["x"][3]
-    __a_model__[i1, i2] = ress_full["x"][1]
-    __b_model__[i1, i2] = ress_full["x"][2]
-    __toa_model__[i1, i2] = model
+        __cwv_model__[i1, i2] = ress_full["x"][0]
+        __cwc_model__[i1, i2] = ress_full["x"][3]
+        __a_model__[i1, i2] = ress_full["x"][1]
+        __b_model__[i1, i2] = ress_full["x"][2]
+        __toa_model__[i1, i2, :] = model
 
 
 # noinspection PyUnresolvedReferences
@@ -895,17 +920,25 @@ def __min_ice__(ii):
 
     i1, i2 = ii
 
-    ress_full = minimize(fun=__forward__, x0=__xa__[i1, i2, :], args=(__pt__[i1, i2, :], __data__[i1, i2, :]),
-                         jac=False, method='Nelder-Mead', options={"disp": False})
+    if not __segmentation__ and __land_only__ and __water_mask__[i1, i2] != 1:
+        __cwv_model__[i1, i2] = np.nan
+        __cwc_model__[i1, i2] = np.nan
+        __ice_model__[i1, i2] = np.nan
+        __a_model__[i1, i2] = np.nan
+        __b_model__[i1, i2] = np.nan
+        __toa_model__[i1, i2, :] = np.full(len(__fit_wvl__), np.nan)
+    else:
+        ress_full = minimize(fun=__forward__, x0=__xa__[i1, i2, :], args=(__pt__[i1, i2, :], __data__[i1, i2, :]),
+                             jac=False, method='Nelder-Mead', options={"disp": False})
 
-    model = __forward__(xx=ress_full["x"], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
+        model = __forward__(xx=ress_full["x"], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
 
-    __cwv_model__[i1, i2] = ress_full["x"][0]
-    __cwc_model__[i1, i2] = ress_full["x"][3]
-    __ice_model__[i1, i2] = ress_full["x"][4]
-    __a_model__[i1, i2] = ress_full["x"][1]
-    __b_model__[i1, i2] = ress_full["x"][2]
-    __toa_model__[i1, i2] = model
+        __cwv_model__[i1, i2] = ress_full["x"][0]
+        __cwc_model__[i1, i2] = ress_full["x"][3]
+        __ice_model__[i1, i2] = ress_full["x"][4]
+        __a_model__[i1, i2] = ress_full["x"][1]
+        __b_model__[i1, i2] = ress_full["x"][2]
+        __toa_model__[i1, i2, :] = model
 
 
 # noinspection PyUnresolvedReferences
@@ -925,39 +958,49 @@ def __oe__(ii):
     else:
         se = np.identity(len(__fit_wvl__)) * ((__data__[i1, i2, :] / __snr__) ** 2)
 
-    res = __inv_func__(yy=__data__[i1, i2, :], fparam=__pt__[i1, i2, :], ll=__ll__, ul=__ul__, xa=__xa__[i1, i2, :],
-                       sa=__sa__, se=se, gnform='n', full=False, maxiter=35, eps=0.01)
+    if not __segmentation__ and __land_only__ and __water_mask__[i1, i2] != 1:
+        __cwv_model__[i1, i2] = np.nan
+        __cwc_model__[i1, i2] = np.nan
+        __a_model__[i1, i2] = np.nan
+        __b_model__[i1, i2] = np.nan
+        __toa_model__[i1, i2, :] = np.full(len(__fit_wvl__), np.nan)
+        __Se__[i1, i2, :, :] = np.full((4, 4), np.nan)
+        __Scem__[i1, i2, :, :] = np.full((4, 4), np.nan)
+        __Srem__[i1, i2, :, :] = np.full((4, 4), np.nan)
+    else:
+        res = __inv_func__(yy=__data__[i1, i2, :], fparam=__pt__[i1, i2, :], ll=__ll__, ul=__ul__, xa=__xa__[i1, i2, :],
+                           sa=__sa__, se=se, gnform='n', full=False, maxiter=35, eps=0.01)
 
-    model = __forward__(xx=res[0], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
+        model = __forward__(xx=res[0], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
 
-    __cwv_model__[i1, i2] = res[0][0]
-    __cwc_model__[i1, i2] = res[0][3]
-    __a_model__[i1, i2] = res[0][1]
-    __b_model__[i1, i2] = res[0][2]
-    __toa_model__[i1, i2, :] = model
+        __cwv_model__[i1, i2] = res[0][0]
+        __cwc_model__[i1, i2] = res[0][3]
+        __a_model__[i1, i2] = res[0][1]
+        __b_model__[i1, i2] = res[0][2]
+        __toa_model__[i1, i2, :] = model
 
-    # a posteriori covariance matrix
-    __Se__[i1, i2, :, :] = res[4]
+        # a posteriori covariance matrix
+        __Se__[i1, i2, :, :] = res[4]
 
-    # correlation error matrix
-    cem = np.zeros((4, 4))
+        # correlation error matrix
+        cem = np.zeros((4, 4))
 
-    for mm in range(4):
-        for nn in range(4):
-            cem[mm, nn] = __Se__[i1, i2, :, :][mm, nn] / np.sqrt(__Se__[i1, i2, :, :][mm, mm] *
-                                                                 __Se__[i1, i2, :, :][nn, nn])
+        for mm in range(4):
+            for nn in range(4):
+                cem[mm, nn] = __Se__[i1, i2, :, :][mm, nn] / np.sqrt(__Se__[i1, i2, :, :][mm, mm] *
+                                                                     __Se__[i1, i2, :, :][nn, nn])
 
-    __Scem__[i1, i2, :, :] = cem
+        __Scem__[i1, i2, :, :] = cem
 
-    # relative error matrix
-    x_ = res[0]
-    rem = np.zeros((4, 4))
+        # relative error matrix
+        x_ = res[0]
+        rem = np.zeros((4, 4))
 
-    for mm in range(4):
-        for nn in range(4):
-            rem[mm, nn] = 100 * np.sqrt(__Se__[i1, i2, :, :][mm, nn] / (x_[mm] * x_[nn]))
+        for mm in range(4):
+            for nn in range(4):
+                rem[mm, nn] = 100 * np.sqrt(__Se__[i1, i2, :, :][mm, nn] / (x_[mm] * x_[nn]))
 
-    __Srem__[i1, i2, :, :] = rem
+        __Srem__[i1, i2, :, :] = rem
 
 
 # noinspection PyUnresolvedReferences
@@ -977,37 +1020,48 @@ def __oe_ice__(ii):
     else:
         se = np.identity(len(__fit_wvl__)) * ((__data__[i1, i2, :] / __snr__) ** 2)
 
-    res = __inv_func__(yy=__data__[i1, i2, :], fparam=__pt__[i1, i2, :], ll=__ll__, ul=__ul__, xa=__xa__[i1, i2, :],
-                       sa=__sa__, se=se, gnform='n', full=False, maxiter=35, eps=0.01)
+    if not __segmentation__ and __land_only__ and __water_mask__[i1, i2] != 1:
+        __cwv_model__[i1, i2] = np.nan
+        __cwc_model__[i1, i2] = np.nan
+        __ice_model__[i1, i2] = np.nan
+        __a_model__[i1, i2] = np.nan
+        __b_model__[i1, i2] = np.nan
+        __toa_model__[i1, i2, :] = np.full(len(__fit_wvl__), np.nan)
+        __Se__[i1, i2, :, :] = np.full((5, 5), np.nan)
+        __Scem__[i1, i2, :, :] = np.full((5, 5), np.nan)
+        __Srem__[i1, i2, :, :] = np.full((5, 5), np.nan)
+    else:
+        res = __inv_func__(yy=__data__[i1, i2, :], fparam=__pt__[i1, i2, :], ll=__ll__, ul=__ul__, xa=__xa__[i1, i2, :],
+                           sa=__sa__, se=se, gnform='n', full=False, maxiter=35, eps=0.01)
 
-    model = __forward__(xx=res[0], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
+        model = __forward__(xx=res[0], pt=__pt__[i1, i2, :], dt=__data__[i1, i2, :], model_output=True)
 
-    __cwv_model__[i1, i2] = res[0][0]
-    __cwc_model__[i1, i2] = res[0][3]
-    __ice_model__[i1, i2] = res[0][4]
-    __a_model__[i1, i2] = res[0][1]
-    __b_model__[i1, i2] = res[0][2]
-    __toa_model__[i1, i2, :] = model
+        __cwv_model__[i1, i2] = res[0][0]
+        __cwc_model__[i1, i2] = res[0][3]
+        __ice_model__[i1, i2] = res[0][4]
+        __a_model__[i1, i2] = res[0][1]
+        __b_model__[i1, i2] = res[0][2]
+        __toa_model__[i1, i2, :] = model
 
-    # a posteriori covariance matrix
-    __Se__[i1, i2, :, :] = res[4]
+        # a posteriori covariance matrix
+        __Se__[i1, i2, :, :] = res[4]
 
-    # correlation error matrix
-    cem = np.zeros((5, 5))
+        # correlation error matrix
+        cem = np.zeros((5, 5))
 
-    for mm in range(5):
-        for nn in range(5):
-            cem[mm, nn] = __Se__[i1, i2, :, :][mm, nn] / np.sqrt(__Se__[i1, i2, :, :][mm, mm] *
-                                                                 __Se__[i1, i2, :, :][nn, nn])
+        for mm in range(5):
+            for nn in range(5):
+                cem[mm, nn] = __Se__[i1, i2, :, :][mm, nn] / np.sqrt(__Se__[i1, i2, :, :][mm, mm] *
+                                                                     __Se__[i1, i2, :, :][nn, nn])
 
-    __Scem__[i1, i2, :, :] = cem
+        __Scem__[i1, i2, :, :] = cem
 
-    # relative error matrix
-    x_ = res[0]
-    rem = np.zeros((5, 5))
+        # relative error matrix
+        x_ = res[0]
+        rem = np.zeros((5, 5))
 
-    for mm in range(5):
-        for nn in range(5):
-            rem[mm, nn] = 100 * np.sqrt(__Se__[i1, i2, :, :][mm, nn] / (x_[mm] * x_[nn]))
+        for mm in range(5):
+            for nn in range(5):
+                rem[mm, nn] = 100 * np.sqrt(__Se__[i1, i2, :, :][mm, nn] / (x_[mm] * x_[nn]))
 
-    __Srem__[i1, i2, :, :] = rem
+        __Srem__[i1, i2, :, :] = rem
