@@ -30,7 +30,7 @@ keyfile: needed in $HOME/.ecmwfapirc
 """
 pip install pygrib # might not work for you since it did not work for me -> manual install might be needed here
 1st: grib_api
-download software here: https://software.ecmwf.int/wiki/display/GRIB/Home
+download software here: https://www.ecmwf.int/en/elibrary/18135-grib-api
 
 e.g. on Fedora 21:
 >yum install openjpeg
@@ -707,8 +707,15 @@ class ECMWF_variable(object):
             self.lat = np.array(h5f["lat"][()][:], dtype=dtype)
             self.lon = np.array(h5f["lon"][()][:], dtype=dtype)
             self.variable = np.array(h5f[variable][()][:, :, :], dtype=dtype)
-            self.var = RegularGridInterpolator(points=(self.steps, self.lat[::-1], self.lon),
-                                               values=self.variable[:, ::-1, :], method="linear")
+            self.points = (self.steps, self.lat[::-1], self.lon)
+            self.values = self.variable[:, ::-1, :]
+            if self.values.shape[0] != len(self.points[0]):
+                self.steps = np.arange(0, self.values.shape[0] * 3, 3, dtype=dtype)
+                self.points = (self.steps, self.lat[::-1], self.lon)
+            for step in range(self.values.shape[0]):
+                if np.isnan(self.values[step, :, :]).any():
+                    self.values[step, :, :] = self.values[step - 1, :, :]
+            self.var = RegularGridInterpolator(points=self.points, values=self.values, method="linear")
 
     def __call__(self, step, lons, lats, shape=None, order=3, dtype=np.float32, force_bounds=True):
         """
@@ -731,7 +738,7 @@ class ECMWF_variable(object):
                 self.logger.warning("Force step to bounds.")
                 step = np.clip(step, self.steps[0], self.steps[-1])
             else:
-                raise ValueError("Parameter step is our of bounds.")
+                raise ValueError("Parameter step is out of bounds.")
 
         ao = self.var(np.vstack((step * np.ones(np.prod(lons.shape)),
                                  lats.reshape(-1),
