@@ -33,6 +33,7 @@ import inspect
 import sys
 import numpy as np
 from numba import jit
+import urllib.request
 
 
 def get_data_file(module_name, file_basename):
@@ -283,3 +284,22 @@ def interpol_lut_c(lut1, lut2, xnodes, nm_nodes, ndim, x_cell, vtest, intp_wvl):
     f_int = np.sum(weights * lut_cell, axis=1)
 
     return f_int
+
+
+def download_LUT(path_LUT_default):
+    """
+    Download LUT file from remote GIT repository.
+
+    :param path_LUT_default: directory where to store the LUT file
+    :return:                 directory of downloaded LUT file
+    """
+    fname = "https://git.gfz-potsdam.de/EnMAP/sicor/-/raw/master/sicor/AC/data/EnMAP_LUT_MOD5_formatted_1nm"
+    urllib.request.urlretrieve(fname, path_LUT_default)
+    if os.path.isfile(path_LUT_default):
+        fn_table = path_LUT_default
+    else:
+        raise FileNotFoundError('Download of LUT file failed. Please download it manually from'
+                                'https://git.gfz-potsdam.de/EnMAP/sicor and store it at /sicor/AC/data/'
+                                'directory. Otherwise, the AC will not work.')
+
+    return fn_table
