@@ -425,7 +425,7 @@ class ClassicalBayesian(object):
                     self.logger.info("class: %s, bins: %i->%i,curr ok: %i, still bad: %i" %
                                      (str(cl), self.n_bins, n_bins, int(np.sum(hh_ok)), int(np.sum(hh_invalid))))
             tt += time() - t0
-        self.logger.info("Time spend in reduced form: %.2f" % tt)
+        self.logger.info("Time spent in reduced form: %.2fs" % tt)
         return pp
 
     def predict_proba(self, xx):
