@@ -67,12 +67,12 @@ class S2Image(RSImage):
         :param granule_path: path to granule folder, folder should contain IMG_DATA folder and S2A_[..].xml file
         :param import_bands: list of bands to import, default: ['B01', 'B02', 'B03', 'B04', 'B05', 'B06', 'B07', 'B08',
                                                                                     'B8A', 'B09', 'B10', 'B11', 'B12',]
-        :param namespace_candidate: for XML file, or None, then several default namaspaces are tested
+        :param namespace_candidate: for XML file, or None, then several default namespaces are tested
         :param target_resolution: spatial resolution in meter, or None for data not interpolated
         :param interpolation_order: integer for interpolation 1,2,3
         :param data_mode: either "dense" or "sparse"
-        :param aux_fields: either None or dict auf to be loaded aus fields, e.g. {"cwv":[ss],"spr":[ss],"ozo":[ss]}
-                           with ss beeing "mean" or iterable of spatial samplings, e.g. [20.0] or [20.0,60.0]
+        :param aux_fields: either None or dict of to be loaded aux fields, e.g. {"cwv":[ss],"spr":[ss],"ozo":[ss]}
+                           with ss being "mean" or iterable of spatial samplings, e.g. [20.0] or [20.0,60.0]
         :param driver: should be "OpenJpeg2000" -> glymur, "kdu_expand"->shell command, "gdal_[driver] e.g. gdal_JP2ECW
                        gdal_JPEG2000, gdal_JP2OpenJPEG,gdal_JP2KAK
 
@@ -257,7 +257,7 @@ class S2Image(RSImage):
                 t2 = time()
                 self.logger.info("""Read band %s in %.2fs, pure load time:%.2fs, resample time: %.2fs, zoom: %.3f,
 final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(self.final_shape), ii))
-        self.logger.info("Total time for reading and converting bands: %.2f" % (time() - _t0))
+        self.logger.info("Total time for reading and converting bands: %.2fs" % (time() - _t0))
 
         if self.target_resolution is None:
             self.band_spatial_sampling = {band: self.metadata['shape_to_resolution'][data.shape]
@@ -330,13 +330,13 @@ final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(
         self.logger.info("Total runtime: %.2fs" % (time() - t0_init))
 
     def get_granule_metadata_xml(self):
-        """Try to find metadat xml file for the granule."""
+        """Try to find metadata xml file for the granule."""
         path_buffer = path.join(self.S2_MSI_granule_path, "MTD_TL.xml")
         if path.exists(path_buffer):
             # new style naming convention
             return path_buffer
         else:
-            # test for oly style naming convention, only one file should match this pattern
+            # test for style naming convention, only one file should match this pattern
             path_buffers = glob(path.join(self.S2_MSI_granule_path, "*_L[12][AC]_TL*.xml"))
             path_buffers = [bf for bf in path_buffers if ".aux.xml" not in bf]  # exclude gdal metadata
 
@@ -420,7 +420,7 @@ final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(
     def parse_s2_granule_xml(fn, namespace):
         """
         parse XML file in granule folder and return metadata
-        :param fn: full filename ti xml file
+        :param fn: full filename tile xml file
         :param namespace: xml namespace
         :return: dictionary with metadata
         """
@@ -567,7 +567,7 @@ final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(
         :param image_data: numpy array
         :return: 2D boolean array
         """
-        self.logger.warning("Masks are computed on value basis, provider mask are not jet implemented")
+        self.logger.warning("Masks are computed on value basis, provider masks are not yet implemented")
         nodata = self.nodata[self.metadata['shape_to_resolution'][image_data.shape]]
         assert nodata.shape == image_data.shape
 
@@ -621,7 +621,7 @@ final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(
         gdal.AllRegister()
         gdal_drivers = [gdal.GetDriver(ii).GetDescription() for ii in range(gdal.GetDriverCount())]
         if driver_name not in gdal_drivers:
-            raise ValueError("Selected driver seems missing in gdal, available are: %s" % str(gdal_drivers))
+            raise ValueError("Selected driver seems to be missing in gdal, available are: %s" % str(gdal_drivers))
         else:
             while gdal.GetDriverCount() > 1:
                 for ii in range(gdal.GetDriverCount()):
@@ -666,7 +666,7 @@ final shape: %s, index: %i""" % (band, t2 - t0, t1 - t0, t2 - t1, zoom_fac, str(
         :rtype: list
         """
         if gt[2] != 0 or gt[4] != 0:  # TODO
-            raise NotImplementedError('Currently rotated datasets are not supported.')
+            raise NotImplementedError('Currently, rotated datasets are not supported.')
         srs = osr.SpatialReference()
         srs.ImportFromWkt(prj)
         proj4 = [i[1:] for i in srs.ExportToProj4().split()]
@@ -712,12 +712,12 @@ class s2_snr_model(object):
     """Sentinel-2 SNR model from CNES instrument characterization.
     :param: s2_snr_csv_file: path to valid s2_snr_csv_file, e.g. line: B1,129.0,588,"0,04840000","0,00003130",
     "4,07374605","129,00","525,5132410906","0,1370807224","941,05","1614,99","983,3"
-    :param: rfl_to_rad: dict with band names as keys and conversion factor from reflectante to radiance, e.g.:
+    :param: rfl_to_rad: dict with band names as keys and conversion factor from reflectance to radiance, e.g.:
             rfl_to_rad={'B8a': 0.0, 'B12': 0.0, 'B10': 0.0, 'B03': 0.0, 'B07': 0.0, 'B04': 0.0, 'B11': 0.0,
             'B02': 0.0, 'B08': 0.0, 'B05': 0.0, 'B09': 0.0, 'B01': 0.0, 'B06': 0.0}
     """
     def __init__(self, s2_snr_csv_file, rfl_to_rad, logger=None):
-        # get glogger
+        # get logger
         self.logger = logging.getLogger() if logger is None else logger
         # read data file
         self.s2_snr_csv_file = s2_snr_csv_file
