@@ -19,6 +19,7 @@ from dicttoxml import dicttoxml
 import matplotlib.pyplot as plt
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 from datetime import datetime, timedelta
+import sys
 
 from .Tools import inpaint, fl, cl, itt2d, nanmean_filter
 from .Tools import write_raster_gdal
@@ -270,25 +271,27 @@ def get_egl_lut(fo, spr_lim, dim_spr, dim_wv):
     return egl_lut
 
 
-def wv(fo, s2img, clear_areas, s2spr, s2tau, nanmean, sigma, rho_bounds,
+def wv(fo, s2img, clear_areas, s2spr, s2tau, cloud_fraction, nanmean, sigma, rho_bounds,
        bands, target_resolution, int_order, n_smpl, n_std, logger=None, **kwargs):
     """
     Generic two-band water vapor retrieval
-    :param fo: Forward operator to use
-    :param s2img: RSImage derived at-sensor reflectance
-    :param clear_areas: Dict of {spatial_sampling:clear_area_map}
-    :param s2spr: Dict {spatial_sampling:surface_pressure_map}
-    :param s2tau: Dict {spatial_sampling:aerosol_optical_thickness_map}
-    :param nanmean: TODO
-    :param sigma: TODO
-    :param rho_bounds: Lower bound for which no retrieval is performed
-    :param bands: TODO
-    :param target_resolution: Value of spatial sampling in meter to retrieve the product
-    :param int_order: Order of polynomial interpolation for output product
-    :param n_smpl: TODO
-    :param n_std: TODO
-    :param logger: None or logging instance
-    :returns: Retrieved water vapor map.
+
+    :param fo:                forward operator to use
+    :param s2img:             RSImage derived at-sensor reflectance
+    :param clear_areas:       dict of {spatial_sampling:clear_area_map}
+    :param s2spr:             dict {spatial_sampling:surface_pressure_map}
+    :param s2tau:             dict {spatial_sampling:aerosol_optical_thickness_map}
+    :param cloud_fraction:    fraction of cloud covered image pixels
+    :param nanmean:           TODO
+    :param sigma:             TODO
+    :param rho_bounds:        lower und upper reflectance bounds
+    :param bands:             TODO
+    :param target_resolution: value of spatial sampling in meter to retrieve the product
+    :param int_order:         order of polynomial interpolation for output product
+    :param n_smpl:            TODO
+    :param n_std:             TODO
+    :param logger:            None or logging instance
+    :returns:                 retrieved water vapor map.
     """
     from scipy.interpolate import griddata  # import here avoids static TLS ImportError
 
@@ -310,8 +313,32 @@ def wv(fo, s2img, clear_areas, s2spr, s2tau, nanmean, sigma, rho_bounds,
     # create field for water vapor
     cwv = np.empty(mask.shape)
     cwv[:, :] = np.nan
-    cwv[mask] = griddata(xi=xi, fill_value=np.NAN, method='linear', **prepare_s2_wv(
-        n_smpl=n_smpl, bands=bands, fo=fo, rho_bounds=rho_bounds))
+    try:
+        cwv[mask] = griddata(xi=xi, fill_value=np.NAN, method='linear', **prepare_s2_wv(
+            n_smpl=n_smpl, bands=bands, fo=fo, rho_bounds=rho_bounds))
+    except RuntimeError:
+        logger.warning("Water vapor retrieval based on initial unstructured grid points failed, "
+                       "most likely due to high cloud fraction of %.3f." % cloud_fraction)
+        logger.info("Try to reduce unstructured grid points to enable a successful water vapor retrieval. "
+                    "This may take a while.")
+        grid = prepare_s2_wv(n_smpl=n_smpl, bands=bands, fo=fo, rho_bounds=rho_bounds)
+        points = grid["points"]
+        values = grid["values"]
+        for ii in np.arange(0, 1.01, 0.01):
+            threshold = points[:, 2].min() + ii
+            try:
+                cwv[mask] = griddata(xi=xi, fill_value=np.NAN, method='linear', points=points[points[:, 2] > threshold],
+                                     values=values[points[:, 2] > threshold])
+                logger.info("Water vapor retrieval succeeded based on reduced unstructured grid points.")
+                break
+            except RuntimeError:
+                if ii == 1.0:
+                    logger.warning("Water vapor retrieval failed even based on reduced unstructured grid points. "
+                                   "Exit AC due to too high cloud cover of input image.")
+                    logger.info("EEooFF")
+                    sys.exit()
+                else:
+                    pass
 
     # filter water vapor retrieval results
     cwv_mean, cwv_std = np.nanmean(cwv), np.nanstd(cwv)
@@ -595,7 +622,13 @@ def ac_interpolation(iband, band, s2img, s2spr, s2cwv, s2tau, clear_areas, point
                         except RuntimeError:
                             pass
                 except (ValueError, IndexError):
-                    res = griddata(points=points, values=rhos, xi=xi)  # old version
+                    for ii in range(points.shape[0]):
+                        index = points.shape[0] - ii
+                        try:
+                            res = griddata(points=points[:index, :], values=rhos[:index], xi=xi)  # old version
+                            break
+                        except RuntimeError:
+                            pass
 
             res[res == 0.0] = np.nan
             data_ac[sx, sy][mask] = res
