@@ -120,7 +120,7 @@ def coreg_s2(granule="", ref_image_fn=None, ref_image_basepath=None, window_size
     :param ref_image_fn: base path for reference image, if None, ref_image is searched in ref_image_basepath
     :param window_size: coreg option
     :param force: True / False, if True, overwrite existing images
-    :param image_pattern: tuple how to find images in products, glop pattern
+    :param image_pattern: tuple how to find images in products, glob pattern
     :return: processing status (0:successful, 1:errors)
     """
     _t0 = time()
@@ -142,7 +142,7 @@ def coreg_s2(granule="", ref_image_fn=None, ref_image_basepath=None, window_size
     try:  # -> e.g. "33UUU"
         tile = re.search('_T[0-9]{2,2}[A-Z]{3,3}_', granule).group(0).replace("_", "")[1:]
     except AttributeError:
-        raise ValueError("Cant get granule name from filename:", granule)
+        raise ValueError("Can't get granule name from filename:", granule)
 
     image_fns = sorted(glob(path.join(granule, image_pattern), recursive=True))
     msk_fns = {img.split("_")[-1].split(".jp2")[0]: img for img in sorted(
@@ -304,7 +304,7 @@ def coreg_s2(granule="", ref_image_fn=None, ref_image_basepath=None, window_size
 
 def cleanup_coreg(granule, logger=None):
     """
-    If an error occures, delete all files containing "coreg" in the granule path.
+    If an error occurs, delete all files containing "coreg" in the granule path.
 
     :param granule: str
     :param logger: logger
@@ -385,7 +385,7 @@ def get_cloud_mask(image_fn):
 # noinspection PyShadowingNames
 def arguments(ignore=("logger",)):
     """
-    Return a tuple containing dictionary of calling function's. named arguments and a list of
+    Return a tuple containing dictionary of calling function's, named arguments and a list of
     calling function's unnamed positional arguments.
     """
     from inspect import getargvalues, stack
@@ -500,7 +500,7 @@ def ac(granule_path, out_dir, settings, aerosol_type="aerosol_1", logger=None, r
                     IO.rename_tmp_product(outputs=options["output"], run_suffix=options["run_suffix"], logger=logger)
 
         else:
-            logger.info("Products already exists -> skip processing.")
+            logger.info("Products already exist -> skip processing.")
             if options["run_suffix"] != "":
                 try:
                     fn = [o["fn"] for o in options["output"] if o["type"] == "L2A"][0]
@@ -567,7 +567,8 @@ def ac(granule_path, out_dir, settings, aerosol_type="aerosol_1", logger=None, r
             for opts in options["output"]:
                 if opts["type"] == "metadata":
                     opts["fn"] = opts["fn"].replace(options["run_suffix"], "")
-                    IO.write_metadata(opts, options, logger=logger)
+                    if os.path.isfile(opts["fn"]):
+                        IO.write_metadata(opts, options, logger=logger)
 
         try:
             if raise_on_memory_exception is True and options["processing"]["Exception_type"] == "<class 'MemoryError'>":
@@ -737,8 +738,14 @@ def ac_gms(s2img, options, logger=None, script=True):
                 logger.info("AC table for scattering type %s was properly downloaded "
                             "and is available for AC!" % scat_type)
             except AssertionError:
+                options["processing"]["status"] = 1
                 raise AssertionError("AC table for scattering type %s was not properly downloaded. "
                                      "Please restart AC and try again." % scat_type)
+            except FileNotFoundError:
+                options["processing"]["status"] = 1
+                raise FileNotFoundError("AC table could not be properly downloaded, the path provided is incorrect. "
+                                        "Please restart AC and try again.")
+
             op['atm_tables_fn'] = default_table_path
 
     # check if cloud mask persistence file is available
@@ -761,8 +768,14 @@ def ac_gms(s2img, options, logger=None, script=True):
             assert os.path.getsize(default_cld_msk_path) == options["cld_mask"]["persistence_file_size"]
             logger.info("Cloud mask persistence file was properly downloaded and is available for AC!")
         except AssertionError:
+            options["processing"]["status"] = 1
             raise AssertionError("Cloud mask persistence file was not properly downloaded. "
                                  "Please restart AC and try again.")
+        except FileNotFoundError:
+            options["processing"]["status"] = 1
+            raise FileNotFoundError("Cloud mask could not be properly downloaded, the path provided is incorrect. "
+                                    "Please restart AC and try again.")
+
         options["cld_mask"]["persistence_file"] = default_cld_msk_path
 
     # check if novelty detector is available
@@ -784,7 +797,13 @@ def ac_gms(s2img, options, logger=None, script=True):
             assert os.path.getsize(default_nov_det_path) == options["cld_mask"]["novelty_detector_size"]
             logger.info("Novelty detector was properly downloaded and is available for AC!")
         except AssertionError:
+            options["processing"]["status"] = 1
             raise AssertionError("Novelty detector was not properly downloaded. Please restart AC and try again.")
+        except FileNotFoundError:
+            options["processing"]["status"] = 1
+            raise FileNotFoundError("Novelty detector could not be properly downloaded, the path provided is incorrect. "
+                                    "Please restart AC and try again.")
+
         options["cld_mask"]["novelty_detector"] = default_nov_det_path
 
     # preprocessing => derive cloud mask
@@ -835,18 +854,37 @@ def ac_gms(s2img, options, logger=None, script=True):
     if options["report"]["reporting"] is True:
         s2rgb = s2img.image_to_rgb(**options["report"]["RGB"])  # make rgb
         s2msk_rgb = s2img.mask_clouds.mask_rgb_array()  # make rgb for mask
+
     # prep suitable masks for processing
-    clear_areas = s2img.mask_clouds.mk_mask_at_spatial_scales(flags=options["AC"]["clear_area_labels"],
-                                                              samplings=s2img.metadata["spatial_samplings"].keys())
+    areas_to_process = s2img.mask_clouds.mk_mask_at_spatial_scales(flags=options["AC"]["labels_to_process"],
+                                                                   samplings=s2img.metadata["spatial_samplings"].keys())
+    clear_pixels = s2img.mask_clouds.mk_mask_at_spatial_scales(flags=["Clear", "Snow", "Water", "Shadow"],
+                                                               samplings=s2img.metadata["spatial_samplings"].keys())
+    cloud_pixels = s2img.mask_clouds.mk_mask_at_spatial_scales(flags=["Cirrus", "Cloud"],
+                                                               samplings=s2img.metadata["spatial_samplings"].keys())
+
     clear_fraction = (
         lambda x: np.sum(x == np.True_) / np.sum(s2img.yesdata[options["cld_mask"]["target_resolution"]]))(
-            clear_areas[options["cld_mask"]["target_resolution"]])
+            clear_pixels[options["cld_mask"]["target_resolution"]])
     if np.isfinite(clear_fraction) == np.False_:  # keep the '==' over 'is', since np.isfinite returns numpy boolean
         clear_fraction = 0.0  # most likely there are no clear areas in the product
-
-    logger.info("Clear fraction: %.4f" % clear_fraction)
+    logger.info("Clear fraction: %.3f" % clear_fraction)
     options["processing"]["clear_fraction"] = clear_fraction
 
+    cloud_fraction = (
+        lambda x: np.sum(x == np.True_) / np.sum(s2img.yesdata[options["cld_mask"]["target_resolution"]]))(
+        cloud_pixels[options["cld_mask"]["target_resolution"]])
+    if np.isfinite(cloud_fraction) == np.False_:  # keep the '==' over 'is', since np.isfinite returns numpy boolean
+        cloud_fraction = 0.0  # most likely there are no clouds in the product
+    logger.info("Cloud fraction: %.3f" % cloud_fraction)
+    options["processing"]["cloud_fraction"] = cloud_fraction
+
+    threshold = options["AC"]["min_clear_fraction"]
+    if clear_fraction < threshold:
+        if "Cloud" or "Cirrus" in options["AC"]["labels_to_process"]:
+            logger.warning("Clear fraction is only %.3f. However, you set 'Cloud' and/or 'Cirrus' as areas to be "
+                           "processed so that AC will continue and also cover cloudy pixels." % clear_fraction)
+
     # calculate novelty fraction
     if hasattr(s2img.mask_clouds, "novelty"):
         if s2img.mask_clouds.novelty is not None:
@@ -854,8 +892,11 @@ def ac_gms(s2img, options, logger=None, script=True):
                 s2img.mask_clouds.novelty.shape)
             logger.info("Novelty fraction is: %.4f" % novelty_fraction)
 
-    # check if clear sky pixel fraction is above predefined threshold, if not, quit AC
-    if clear_fraction < options["AC"]["min_clear_fraction"]:
+    # check if fraction of pixels to be processed is above predefined threshold, if not, quit AC
+    process_fraction = (
+        lambda x: np.sum(x == np.True_) / np.sum(s2img.yesdata[options["cld_mask"]["target_resolution"]]))(
+        areas_to_process[options["cld_mask"]["target_resolution"]])
+    if process_fraction < options["AC"]["min_clear_fraction"]:
         logger.info("Clear fraction is below %.2f -> quit here." % options["AC"]["min_clear_fraction"])
         if options["report"]["reporting"] is True:
             figs = Figs(s2rgb=s2rgb, s2dem=s2img.dem, s2msk_rgb=s2msk_rgb, logger=logger)
@@ -867,13 +908,13 @@ def ac_gms(s2img, options, logger=None, script=True):
                 dpi=options["report"]["dpi"], n_cols=options["report"]["n_cols"], figs=options["report"]["figs_clouds"])
 
         if script is True:
-            # write cloud fraction to log file
+            # write clear fraction to log file
             for opts in options["output"]:
                 fn_log = opts["fn"] + ".log"
-                logger.info("Write cloud fraction to:%s" % fn_log)
+                logger.info("Write clear fraction to:%s" % fn_log)
                 with open(fn_log, "w") as fl:
                     fl.writelines("Clear fraction %.2f is below %.2f -> quit here." % (
-                        clear_fraction, options["AC"]["min_clear_fraction"]))
+                        process_fraction, options["AC"]["min_clear_fraction"]))
             logger.info("EEooFF")
             sys.exit()
         else:
@@ -906,8 +947,8 @@ def ac_gms(s2img, options, logger=None, script=True):
         del slp
         del t2m
     except Exception as err:
-        logger.info("Error occurred: %s" % str(repr(err)))
-        logger.info("NWP data is missing -> proceed with standard values.")
+        logger.warning("ECMWF surface pressure is missing -> proceed with standard values calculated from DEM.")
+        logger.warning("Error message was: %s" % str(repr(err)))
         spr = np.array(barometric_formula(h=s2img.dem), dtype=np.float16)
     s2spr = {options["ECMWF"]["target_resolution"]: spr}
     del spr
@@ -920,31 +961,33 @@ def ac_gms(s2img, options, logger=None, script=True):
     try:
         mean_ozo = s2img.aux_fields["ozo"] * options["ozo_to_DU"]
         logger.info("Got Ozone from metadata.")
-    except Exception:
+    except AttributeError:
         try:
             logger.info("Ozone missing in metadata -> fallback to ECMWF.")
             mean_ozo = np.mean(options["ozo_to_DU"] * get_ecmwf_data(variable="fc_O3", **ecmwf_opts))
             logger.info("Mean Ozone value: %.2f" % mean_ozo)
         except Exception as err:
+            logger.warning("ECMWF ozone value is missing -> proceed with default value.")
             mean_ozo = options["ozone_fallback"]
-            logger.info("Error when retrieving Ozone: %s" % str(repr(err)))
-            logger.info("Error with ECMWF data -> fallback to default value: %.2f" % mean_ozo)
-            logger.info("Error:%s" % str(repr(err)))
+            logger.info("Default ozone value: %.2f DU" % mean_ozo)
+            logger.error("Error message was: %s" % str(repr(err)))
 
     # get sun and viewing angles
     mean_sza = s2img.metadata['sun_mean_zenith']
 
     if "values" in dir(s2img.metadata["viewing_zenith"]):
-        mean_vza = np.mean([np.nanmean(val) for val in s2img.metadata["viewing_zenith"].values()])
+        mean_vza = np.mean([np.nanmean(val.astype(np.float32))
+                            for val in s2img.metadata["viewing_zenith"].values()])
     else:
         mean_vza = float(s2img.metadata["viewing_zenith"])
     if not np.isfinite(mean_vza):
         mean_vza = 0.0
 
     if "values" in dir(s2img.metadata["viewing_azimuth"]):
-        mean_azi = (s2img.metadata["sun_mean_azimuth"] - (np.nanmean(
-            [np.nanmean(np.array(val, dtype=np.float32)) for val in s2img.metadata["viewing_azimuth"].values()])
-        ) % 180.0)
+        mean_azi = (s2img.metadata["sun_mean_azimuth"]
+                    - (np.nanmean([np.nanmean(val.astype(np.float32))
+                                   for val in s2img.metadata["viewing_azimuth"].values()]))
+                    % 180.0)
     else:
         mean_azi = s2img.metadata["sun_mean_azimuth"] - (float(s2img.metadata["viewing_azimuth"]) % 180.0)
     if not np.isfinite(mean_azi):
@@ -987,14 +1030,16 @@ def ac_gms(s2img, options, logger=None, script=True):
         else:
             aerosol_type = options["AC"]["default_aerosol_type"]
             logger.info("Fallback to default aerosol type: %s." % aerosol_type)
-    except Exception:
+    except Exception as err:
         # ECMWF data not available -> proceed with fallback value and type
-        logger.warning("ECMWF Data missing -> proceed with default type.")
-        logger.warning("Print traceback in case you care:")
-        logger.error(traceback.format_exc())
-        s2tau = {smpl: 0.1 * np.ones(field.shape, dtype=np.float16) for smpl, field in s2spr.items()}
+        logger.warning("ECMWF AOT value is missing -> proceed with default value and type.")
+        default_aot = options["AC"]["default_aot_value"]
         aerosol_type = options["AC"]["default_aerosol_type"]
-        logger.info("Aerosol type: %s" % aerosol_type)
+        logger.info("Default AOT value: %.2f" % default_aot)
+        logger.info("Default aerosol type: %s" % aerosol_type)
+        s2tau = {smpl: np.full(field.shape, default_aot, dtype=np.float16)
+                 for smpl, field in s2spr.items()}
+        logger.error("Error message was: %s" % str(repr(err)))
 
     # #######################
     # ## sensor definition ##
@@ -1051,25 +1096,31 @@ def ac_gms(s2img, options, logger=None, script=True):
         fo.set_rho_lin(Rho2Rj=Rho2Rj_const(rho_wvl=fo.wvl_sensors[sensor_wv]),
                        Rj2RhoJ=Rj2RhoJ_const(rho_wvl=fo.wvl_sensors[sensor_wv]))
         fo.interpolation_settings(jacobean=False, caching=False)
-        s2cwv = wv(fo=fo, s2img=s2img, s2spr=s2spr, s2tau=s2tau, clear_areas=clear_areas,
-                   logger=logger, **options["water_vapor"])
-
-        # s2cwv = wv_band_ratio(fo=fo, s2img=s2img, s2spr=s2spr, clear_areas=clear_areas,
-        # logger=logger, **options["water_vapor"])
+        s2cwv = wv(fo=fo, s2img=s2img, s2spr=s2spr, s2tau=s2tau, clear_areas=areas_to_process,
+                   cloud_fraction=cloud_fraction, logger=logger, **options["water_vapor"])
 
         # fallback to ECMWF:
         finite_test = (lambda x: [np.isfinite(np.nanmin(x)), np.isfinite(np.nanmax(x))])(s2cwv[sorted(s2cwv.keys())[0]])
         if np.False_ in finite_test:
-            logger.info("CWV values are not valid -> fallback to ECMWF")
+            logger.info("CWV values are not valid -> fallback to ECMWF.")
 
             s2cwv = {options["ECMWF"]["target_resolution"]: get_ecmwf_data(
                 variable=options["ECMWF"]["variable_tcwv"], **ecmwf_opts)}
             include_missing_spatial_samplings(data=s2cwv, samplings=s2img.metadata["spatial_samplings"].keys())
 
     elif options["water_vapor"]["type"] == "ECMWF":
-        s2cwv = {options["ECMWF"]["target_resolution"]: get_ecmwf_data(
-            variable=options["ECMWF"]["variable_tcwv"], **ecmwf_opts)}
-        include_missing_spatial_samplings(data=s2cwv, samplings=s2img.metadata["spatial_samplings"].keys())
+        try:
+            s2cwv = {options["ECMWF"]["target_resolution"]: get_ecmwf_data(
+                variable=options["ECMWF"]["variable_tcwv"], **ecmwf_opts)}
+            include_missing_spatial_samplings(data=s2cwv, samplings=s2img.metadata["spatial_samplings"].keys())
+        except Exception as err:
+            # ECMWF data not available -> proceed with default value
+            logger.warning("ECMWF CWV value is missing -> proceed with default value.")
+            default_cwv = options["AC"]["default_cwv_value"]
+            logger.info("Default CWV value: %.2f mm" % default_cwv)
+            s2cwv = {smpl: np.full(field.shape, default_cwv, dtype=np.float16)
+                     for smpl, field in s2spr.items()}
+            logger.error("Error message was: %s" % str(repr(err)))
     else:
         raise ValueError('options["water_vapor"]["type"]=%s is not implemented' % str(options["water_vapor"]["type"]))
 
@@ -1095,11 +1146,11 @@ def ac_gms(s2img, options, logger=None, script=True):
                    Rj2RhoJ=Rj2RhoJ_const(rho_wvl=fo.wvl_sensors[sensor_ac]))
     fo.interpolation_settings(jacobean=False, caching=False)
     points, rhos, rfls = prepare_s2_ac(
-        s2spr=s2spr, s2cwv=s2cwv, s2tau=s2tau, clear_areas=clear_areas, fo=fo, **options["AC"])
+        s2spr=s2spr, s2cwv=s2cwv, s2tau=s2tau, clear_areas=areas_to_process, fo=fo, **options["AC"])
 
     ac_kw = {"points": points, "rhos": rhos, "rfls": rfls,
              "max_pixel_processing": options["AC"]["max_pixel_processing"],
-             's2img': s2img, 's2spr': s2spr, 's2cwv': s2cwv, 's2tau': s2tau, 'clear_areas': clear_areas}
+             's2img': s2img, 's2spr': s2spr, 's2cwv': s2cwv, 's2tau': s2tau, 'clear_areas': areas_to_process}
 
     logger.info("Start ac processing using %i processes." % options["AC"]["n_cores"])
     if options["AC"]["n_cores"] == 1:
@@ -1155,7 +1206,7 @@ def ac_gms(s2img, options, logger=None, script=True):
     logger.info("Append results to input object.")
 
     s2img.wvl_ac = fo.wvl_sensors[sensor_ac]
-    s2img.clear_areas = clear_areas
+    s2img.clear_areas = areas_to_process
     if "Cirrus" in s2img.mask_clouds.mask_legend.keys():
         s2img.cirrus_areas = cirrus_areas
     s2img.data_ac = data_ac
@@ -1172,7 +1223,7 @@ def ac_gms(s2img, options, logger=None, script=True):
             100 * np.sum(copy_area) / np.prod(copy_area.shape)))
 
     # remove tau, cwv, and spr for nonclear areas from final products
-    for ss, clear_area in clear_areas.items():
+    for ss, clear_area in areas_to_process.items():
         s2tau[ss][np.logical_not(clear_area)] = np.nan
         s2cwv[ss][np.logical_not(clear_area)] = np.nan
         s2spr[ss][np.logical_not(clear_area)] = np.nan
